#include <ESP8266WiFi.h>          // https://github.com/esp8266/Arduino

#include <DNSServer.h>
#include <ESP8266WebServer.h>
#include "WiFiManager.h"         // https://github.com/tzapu/WiFiManager

#include <PubSubClient.h>

char mqtt_server[128] = "192.168.64.1";

#define pinLed 5
#define pinPulse 4

WiFiManagerParameter custom_mqtt_server("server", "mqtt server", mqtt_server, 40);

static ESP8266WebServer *server = NULL;

void configModeCallback(WiFiManager *myWiFiManager) {
	Serial.println(F("Entered config mode"));
	Serial.println(WiFi.softAPIP());
	Serial.println(myWiFiManager->getConfigPortalSSID());
}

volatile uint16_t detectedCount = 0;
volatile uint32_t latestEvent = 0;

void ICACHE_RAM_ATTR ifunc() {
	detectedCount++;

	digitalWrite(pinLed, HIGH);
	latestEvent = millis();
}

void handleRoot() {
	char buffer[128] = "?";
	snprintf(buffer, sizeof buffer, "%ld", detectedCount);
	buffer[sizeof buffer - 1] = 0x00;

	server->send(200, "text/plain", buffer);
}

void reboot() {
	Serial.println(F("Reboot"));
	Serial.flush();
	delay(100);
	ESP.restart();
	delay(1000);
}

void setupWifi() {
	WiFi.hostname("GEIGERMEEE");
	WiFi.begin();

	WiFiManager wifiManager;
	wifiManager.setTimeout(120);
	wifiManager.setAPCallback(configModeCallback);
	wifiManager.addParameter(&custom_mqtt_server);

	if (!wifiManager.autoConnect("GeigerMeee"))
		reboot();
}

WiFiClient wclient;
PubSubClient client(wclient);

void reconnect() {
	// Loop until we're reconnected
	while (!client.connected()) {
		Serial.print("Attempting MQTT connection... ");
		Serial.println(mqtt_server);

		char name[32];
		snprintf(name, sizeof name, "geigermeee-%d", ESP.getChipId());

		// Attempt to connect
		if (client.connect(name)) {
			Serial.println(F("Connected"));
			client.publish("geigermeee-meta", "values in uSv");
			break;
		}

		reboot();
	}
}

IPAddress parseIpStr(const char *str) {
	uint8_t a, b, c, d;

	char *dot = strchr(str, '.');
	if (dot) {
		*dot = 0x00;
		a = atoi(str);
		b = atoi(dot + 1);
		dot = strchr(dot + 1, '.');
		if (dot)  {
			*dot = 0x00;
			c = atoi(dot + 1);
			dot = strchr(dot + 1, '.');
			if (dot)
				d = atoi(dot + 1);
		}
	}

	return IPAddress(a, b, c, d);
}

void setup() {
	Serial.begin(115200);
	Serial.println(F("Init radiation mqtt"));

	pinMode(pinLed, OUTPUT);

	setupWifi();

	IPAddress s = parseIpStr(mqtt_server);
	client.setServer(s, 1883);

	Serial.println(F("Go!"));
	server = new ESP8266WebServer(80);
	server->on("/", handleRoot);

	server->begin();

	reconnect();

	pinMode(pinPulse, INPUT);
	attachInterrupt(digitalPinToInterrupt(pinPulse), ifunc, RISING);
}

unsigned long int start = millis();

void loop() {
	for(;;) {
		client.loop();
		server->handleClient();

		unsigned long int now = millis();
		unsigned long int waited = now - start;
		if (waited >= 60l * 1000l)
			break;

		int left = 60l * 1000l - waited;

		if (latestEvent != 0 && now - latestEvent >= 10) {
			latestEvent = 0;
			digitalWrite(pinLed, LOW);
		}
	}

	uint32_t v = detectedCount;
	detectedCount = 0;
	start = millis();

	v *= 1000;
	v /= 151;

	char buffer[64] { 0 };
	snprintf(buffer, sizeof buffer, "%d.%03d", int(v / 1000), int(v % 1000));

	Serial.print(F("Publish: "));
	Serial.println(buffer);

	static byte tries = 0;
	if (!client.publish("geigermeee", buffer)) {
		Serial.println(F("Publish failed"));
		client.disconnect();
		reconnect();

		if (++tries >= 5) {
			Serial.println(F("Reboot"));
			reboot();
		}
	}
	else {
		tries = 0;
	}
}
